/**
* Header file of ML CFP2 API library.
*/
#pragma once
#ifndef ML_CFP2_H__
#define ML_CFP2_H__

#ifndef APICALL
#ifdef __cplusplus
#   ifdef WIN32
#       ifndef ML_IMPORTS_API
#           define APICALL extern "C" __declspec(dllexport)
#       else
#           define APICALL extern "C" __declspec(dllimport)
#       endif
#   else
#       define APICALL extern "C" 
#   endif // WIN32
#else
#   define  APICALL
#endif
#endif
#ifdef __cplusplus
#define  CFP2STACKMODE   __stdcall
#else
#define  CFP2STACKMODE
#endif

#define MAXDEVNUM   32      // Maximum number of attached devices

enum { MLCFP2WRAPPERVERSION = 1 };  // Incremental module version

struct mlcfp2;  // ML CFP2 API wrapper structure.

typedef     unsigned char	UInt8;
typedef     unsigned short	UInt16;
typedef     unsigned long	UInt32;

enum DEVICETYPE
{
    ML_CFP2,
    ML_CFP2_FTDI
};

/// <summary>Enumerate attached devices.</summary>
/// <param name="devices">
/// Output array of detected instance numbers.
/// Array's expected length is `MAXDEVNUM`.
/// </param>
/// <returns>Number of detected devices.</param>
APICALL int CFP2STACKMODE mlcfp2_enumerateDevices(DEVICETYPE type, UInt16 devices[], UInt8 serialNum[]);

/// <summary>Create a CFP2 API wrapper instance.</summary>
/// <param name="instNum">USB instance number.</param>
/// <returns>Pointer to created API wrapper.</returns>
APICALL mlcfp2* CFP2STACKMODE mlcfp2_createInstance(DEVICETYPE type, UInt16 instNum);

/// <summary>Destroy a CFP2 API wrapper instance.</summary>
/// <param name="inst">Wrapper instance.</param>
APICALL void CFP2STACKMODE mlcfp2_destroyInstance(mlcfp2* inst);

/// <summary>Connect to an instance.</summary>
/// <param name="inst">Wrapper instance.</param>
APICALL bool CFP2STACKMODE mlcfp2_connect(mlcfp2* inst);

/// <summary>Disconnect to an instance.</summary>
/// <param name="inst">Wrapper instance.</param>
APICALL bool CFP2STACKMODE mlcfp2_disconnect(mlcfp2* inst);

/// <summary>Get HW info.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="inBuff">Buffer to get info.</param>
APICALL int CFP2STACKMODE mlcfp2_ping(mlcfp2* inst, UInt8 *inBuff);

/// <summary>Read EEPROM.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="address">EEPROM Address.</param>
/// <param name="Value">EEPROM Value.</param>
APICALL int CFP2STACKMODE mlcfp2_eepromRead(mlcfp2* inst, UInt32 address, UInt8 *Value);

/// <summary>Write EEPROM.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="address">EEPROM Address.</param>
/// <param name="Value">EEPROM Value.</param>
APICALL int CFP2STACKMODE mlcfp2_eepromWrite(mlcfp2* inst, UInt32 address, UInt8 Value);

/// <summary>Reset MCU.</summary>
/// <param name="inst">Wrapper instance.</param>
APICALL int CFP2STACKMODE mlcfp2_resetMCU(mlcfp2* inst);

/// <summary>Reset MCU Boot.</summary>
/// <param name="inst">Wrapper instance.</param>
APICALL int CFP2STACKMODE mlcfp2_resetMCUBoot(mlcfp2* inst);

/// <summary>Sets the value of a Microcontroller Port.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="t_Port">Port Number.</param>
/// <param name="Value">Port Value.</param>
APICALL int CFP2STACKMODE mlcfp2_setPort(mlcfp2* inst, UInt8 t_Port, UInt8 Value);

/// <summary>Reads Microcontroller Port.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="t_Port">Port Number.</param>
/// <param name="Value">Port Value.</param>
APICALL int CFP2STACKMODE mlcfp2_getPort(mlcfp2* inst, UInt8 t_Port, UInt8* Value);

/// <summary>Control Vendor IO pins (SW/HW).</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="Value">SW = 0; HW = 1</param>
APICALL int CFP2STACKMODE mlcfp2_setVendorIO(mlcfp2* inst, UInt8 Value);

/// <summary>Enable Current Sense.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="threshold">Current threshold.</param>
/// <param name="delay"> delay.</param>
APICALL int CFP2STACKMODE mlcfp2_enableCurrentSense(mlcfp2* inst, UInt16 threshold, UInt16 delay);

/// <summary>Get Current Sense.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="RegisterAddress">Current Register Address.</param>
/// <param name="Data">Current Value.</param>
APICALL int CFP2STACKMODE mlcfp2_getCurrentSense(mlcfp2* inst, UInt16 RegisterAddress, UInt8 *Data);

/// <summary>Current Monitor</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="Data">Current Value.</param>
APICALL int CFP2STACKMODE mlcfp2_currentMonitor(mlcfp2* inst, UInt16 *Data);

/// <summary>Set MDIO.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="RegisterAddress">Register Address.</param>
/// <param name="Data">Data value.</param>
APICALL int CFP2STACKMODE mlcfp2_setMDIO(mlcfp2* inst, UInt8 DeviceAddress, UInt16 RegisterAddress, UInt16 Data);

/// <summary>Get MDIO.</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="RegisterAddress">Register Address.</param>
/// <param name="Data">Data reference.</param>
APICALL int CFP2STACKMODE mlcfp2_getMDIO(mlcfp2* inst, UInt8 DeviceAddress, UInt16 RegisterAddress, UInt16 *Data);

/// <summary>Get Host VCC (FTDI only).</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="Data">Voltage Data reference.</param>
APICALL int CFP2STACKMODE mlcfp2_getVCC(mlcfp2* inst, double *Data);

/// <summary>Get Host Temperature (FTDI only) .</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="select">Selected Temperature.</param>
/// <param name="Data">Data reference.</param>
APICALL int CFP2STACKMODE mlcfp2_getTemperature(mlcfp2* inst, int select, double *Data);

/// <summary>Enable FTDI (FTDI only) .</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="isEnabled">FTDI pin state.</param>
APICALL int CFP2STACKMODE mlcfp2_p3v3_FTDIEnable(mlcfp2* inst, bool isEnabled);

/// <summary>Set voltage regulator output. (FTDI only) .</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="rate">0 fo 3.15V output, 1 for 3.3V and 2 for 3.45V.</param>
APICALL int CFP2STACKMODE mlcfp2_setPower(mlcfp2* inst, UInt8 rate);

/// <summary>Gets Device SN  (Needs a FW update).</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="serialNum">SN returned value.</param>>
APICALL bool CFP2STACKMODE mlcfp2_getSN(mlcfp2* inst, UInt8 serialNum[]);

/// <summary>Sets a Device SN  (Needs a FW update).</summary>
/// <param name="inst">Wrapper instance.</param>
/// <param name="serialNum">SN Value.</param>>
APICALL bool CFP2STACKMODE mlcfp2_setSN(mlcfp2* inst, UInt8 serialNum[], int SNsize);

//APICALL bool CFP2STACKMODE mlcfp2_setMDIOWS(mlcfp2* inst, UInt8 DeviceAddress, UInt16 RegisterAddress, UInt16 Data);

//APICALL bool CFP2STACKMODE mlcfp2_getMDIOINC(mlcfp2* inst, UInt8 DeviceAddress, UInt16 RegisterAddress, UInt16 *Data);

#endif // !ML_CFP2_H__

